@@ -29,6 +29,7 @@
 #include <qclipboard.h>
 #include <qcolor.h>
 #include <qscrollbar.h>
+#include <qtextcodec.h>
 
 #include <qdragobject.h>
 #include <qevent.h>
@@ -100,7 +101,8 @@ QsciScintillaBase::QsciScintillaBase(QWidget *parent, const char *name,
 
     setFocusPolicy(WheelFocus);
 
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC)
+// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
+#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
     RectangularPasteboardMime::initialise();
 #endif
 
@@ -142,6 +144,14 @@ QsciScintillaBase *QsciScintillaBase::pool()
 }
 
 
+// Tell Scintilla to update the scroll bars.  Scintilla should be doing this
+// itself.
+void QsciScintillaBase::setScrollBars()
+{
+    sci->SetScrollBars();
+}
+
+
 // Return the horizontal scrollbar widget.
 QScrollBar *QsciScintillaBase::horizontalScrollBar() const
 {
@@ -427,7 +437,6 @@ void QsciScintillaBase::handleSelection()
 void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 {
     unsigned key, modifiers = 0;
-    QCString utf8;
 
     if (e->state() & Qt::ShiftButton)
         modifiers |= SCMOD_SHIFT;
@@ -519,27 +528,7 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         break;
 
     default:
-        // See if the input was a single ASCII key.  If so it will be passed to
-        // KeyDownWithModifiers to allow it to be filtered.  Correct the
-        // modifiers and key for ASCII letters as Qt uses the ASCII code of
-        // uppercase letters for Key_A etc.
-        utf8 = e->text().utf8();
-
-        if (utf8.length() == 0)
-            key = e->key();
-        else if (utf8.length() != 1)
-            key = 0;
-        else if ((key = utf8[0]) >= 0x80)
-            key = 0;
-        else if (key >= 0x01 && key <= 0x1a)
-            key += 0x40;
-        else if (key >= 'A' && key <= 'Z')
-            modifiers |= true;
-        else if (key >= 'a' && key <= 'z')
-        {
-            key -= 0x20;
-            modifiers &= ~SCMOD_SHIFT;
-        }
+        key = e->key();
     }
 
     if (key)
@@ -555,15 +544,33 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         }
     }
 
-    // Add the text if it has a compatible size depending on what Unicode mode
-    // we are in.
-    if (utf8.length() > 0 && (sci->IsUnicodeMode() || utf8.length() == 1))
+    QString text = e->text();
+
+    if (!text.isEmpty() && text[0].isPrint())
     {
-        sci->AddCharUTF(utf8.data(), utf8.length());
+        QCString enc_text;
+
+        if (sci->IsUnicodeMode())
+        {
+            enc_text = text.utf8();
+        }
+        else
+        {
+            static QTextCodec *codec = 0;
+
+            if (!codec)
+                codec = QTextCodec::codecForName("ISO 8859-1");
+
+            enc_text = codec->fromUnicode(text);
+        }
+
+        sci->AddCharUTF(enc_text.data(), enc_text.length());
         e->accept();
     }
     else
+    {
         QWidget::keyPressEvent(e);
+    }
 }
 
 
