@@ -29,6 +29,7 @@
 #include <qclipboard.h>
 #include <qcolor.h>
 #include <qscrollbar.h>
+#include <qtextcodec.h>
 
 #include <QContextMenuEvent>
 #include <QDragEnterEvent>
@@ -83,7 +84,8 @@ static const QLatin1String mimeRectangularWin("MSDEVColumnSelect");
 static const QLatin1String mimeRectangular("text/x-qscintilla-rectangular");
 static const QLatin1String utiRectangularMac("com.scintilla.utf16-plain-text.rectangular");
 
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC)
+// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
+#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
 #include <QMacPasteboardMime>
 
 class RectangularPasteboardMime : public QMacPasteboardMime
@@ -178,7 +180,8 @@ QsciScintillaBase::QsciScintillaBase(QWidget *parent)
 
     triple_click.setSingleShot(true);
 
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC)
+// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
+#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
     RectangularPasteboardMime::initialise();
 #endif
 
@@ -220,6 +223,14 @@ QsciScintillaBase *QsciScintillaBase::pool()
 }
 
 
+// Tell Scintilla to update the scroll bars.  Scintilla should be doing this
+// itself.
+void QsciScintillaBase::setScrollBars()
+{
+    sci->SetScrollBars();
+}
+
+
 // Send a message to the real Scintilla widget using the low level Scintilla
 // API.
 long QsciScintillaBase::SendScintilla(unsigned int msg, unsigned long wParam,
@@ -408,7 +419,6 @@ void QsciScintillaBase::handleSelection()
 void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 {
     unsigned key, modifiers = 0;
-    QByteArray utf8;
 
     if (e->modifiers() & Qt::ShiftModifier)
         modifiers |= SCMOD_SHIFT;
@@ -500,27 +510,7 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         break;
 
     default:
-        // See if the input was a single ASCII key.  If so it will be passed to
-        // KeyDownWithModifiers to allow it to be filtered.  Correct the
-        // modifiers and key for ASCII letters as Qt uses the ASCII code of
-        // uppercase letters for Key_A etc.
-        utf8 = e->text().toUtf8();
-
-        if (utf8.length() == 0)
-            key = e->key();
-        else if (utf8.length() != 1)
-            key = 0;
-        else if ((key = utf8[0]) >= 0x80)
-            key = 0;
-        else if (key >= 0x01 && key <= 0x1a)
-            key += 0x40;
-        else if (key >= 'A' && key <= 'Z')
-            modifiers |= true;
-        else if (key >= 'a' && key <= 'z')
-        {
-            key -= 0x20;
-            modifiers &= ~SCMOD_SHIFT;
-        }
+        key = e->key();
     }
 
     if (key)
@@ -536,15 +526,33 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         }
     }
 
-    // Add the text if it has a compatible size depending on what Unicode mode
-    // we are in.
-    if (utf8.length() > 0 && (sci->IsUnicodeMode() || utf8.length() == 1))
+    QString text = e->text();
+
+    if (!text.isEmpty() && text[0].isPrint())
     {
-        sci->AddCharUTF(utf8.data(), utf8.length());
+        QByteArray enc_text;
+
+        if (sci->IsUnicodeMode())
+        {
+            enc_text = text.toUtf8();
+        }
+        else
+        {
+            static QTextCodec *codec = 0;
+
+            if (!codec)
+                codec = QTextCodec::codecForName("ISO 8859-1");
+
+            enc_text = codec->fromUnicode(text);
+        }
+
+        sci->AddCharUTF(enc_text.data(), enc_text.length());
         e->accept();
     }
     else
+    {
         QAbstractScrollArea::keyPressEvent(e);
+    }
 }
 
 
